@@ -14,12 +14,24 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+import sys
 import urllib2
 import re
 import time
 from os import path
 from calculate.lib.configparser import ConfigParserCaseSens, Error as CPError
 from calculate.lib.utils.tools import SingletonParam
+from .files import writeFile, xz
+from collections import MutableMapping
+
+_ = lambda x: x
+from calculate.lib.cl_lang import setLocalTranslate
+
+setLocalTranslate('cl_lib3', sys.modules[__name__])
+
+MINUTES = 60
+HOURS = 60 * MINUTES
+DAYS = 24 * HOURS
 
 
 class BinhostsBase(object):
@@ -159,6 +171,73 @@ class BinhostsBase(object):
                               ",".join("%s:%s" % (str(k), str(v)) for k, v in
                                        kw.items()))
 
+class BinhostError(Exception):
+    pass
 
 class Binhosts(BinhostsBase):
     __metaclass__ = SingletonParam
+
+class PackagesIndex(MutableMapping):
+    def __init__(self, data):
+        header, self.body = data.partition("\n\n")[::2]
+        self.header_dict = {}
+        for line in header.split('\n'):
+            k, v = line.partition(":")[::2]
+            self.header_dict[k] = v.strip()
+
+    def __getitem__(self, item):
+        return self.header_dict[item]
+
+    def __iter__(self):
+        return iter(self.header_dict)
+
+    def __len__(self):
+        return len(self.header_dict)
+
+    def __setitem__(self, key, value):
+        self.header_dict[key] = value
+
+    def __delitem__(self, key):
+        self.header_dict.pop(key)
+
+    def write(self, f):
+        f.write("\n".join("%s: %s" % (x, self.header_dict[x])
+                          for x in sorted(self.header_dict.keys())))
+        f.write("\n\n%s"%self.body)
+
+def fetch_packages(url_binhost, cache_fn, timeout=300, ttl=30 * DAYS):
+    """
+    Получить файл Packages из бинарного хоста (распаковать если архив,
+    добавить поля DOWNLOAD_TIMESTAMP и TTL
+    :param url_binhost:
+    :param cache_fn:
+    :param timeout:
+    :param ttl:
+    :return:
+    """
+    data = None
+    for uri in ("Packages.xz", "Packages"):
+        fn = path.join(url_binhost, uri)
+        try:
+            data = urllib2.urlopen(fn, timeout=timeout).read()
+            if uri == "Packages.xz":
+                data = xz(data, decompress=True)
+                break
+        except urllib2.URLError as e:
+            pass
+        except BaseException as e:
+            if isinstance(e, KeyboardInterrupt):
+                raise
+            data = ""
+    if not data:
+        raise BinhostError(_("Failed to fetch Packages from binary host %s")
+                           %url_binhost)
+    pi = PackagesIndex(data)
+    pi["TTL"] = str(ttl)
+    pi["DOWNLOAD_TIMESTAMP"] = str(int(time.time()))
+    try:
+        with writeFile(cache_fn) as f:
+            pi.write(f)
+    except (OSError, IOError):
+        raise BinhostError(_("Failed to save Packages"))
+    return True
